@@ -1,13 +1,17 @@
 package Catalyst::Utils;
 
 use strict;
-use Catalyst::Exception;
 use File::Spec;
 use HTTP::Request;
 use Path::Class;
 use URI;
 use Carp qw/croak/;
 use Cwd;
+use Class::MOP;
+use String::RewritePrefix;
+use Class::Load ();
+
+use namespace::clean;
 
 =head1 NAME
 
@@ -19,7 +23,7 @@ See L<Catalyst>.
 
 =head1 DESCRIPTION
 
-Catalyst Utilities. 
+Catalyst Utilities.
 
 =head1 METHODS
 
@@ -120,7 +124,7 @@ sub class2prefix {
 Returns a tempdir for a class. If create is true it will try to create the path.
 
     My::App becomes /tmp/my/app
-    My::App::C::Foo::Bar becomes /tmp/my/app/c/foo/bar
+    My::App::Controller::Foo::Bar becomes /tmp/my/app/c/foo/bar
 
 =cut
 
@@ -136,6 +140,13 @@ sub class2tempdir {
         eval { $tmpdir->mkpath };
 
         if ($@) {
+            # don't load Catalyst::Exception as a BEGIN in Utils,
+            # because Utils often gets loaded before MyApp.pm, and if
+            # Catalyst::Exception is loaded before MyApp.pm, it does
+            # not honor setting
+            # $Catalyst::Exception::CATALYST_EXCEPTION_CLASS in
+            # MyApp.pm
+            require Catalyst::Exception;
             Catalyst::Exception->throw(
                 message => qq/Couldn't create tmpdir '$tmpdir', "$@"/ );
         }
@@ -148,8 +159,31 @@ sub class2tempdir {
 
 Returns home directory for given class.
 
+=head2 dist_indicator_file_list
+
+Returns a list of files which can be tested to check if you're inside
+a CPAN distribution which is not yet installed.
+
+These are:
+
+=over
+
+=item Makefile.PL
+
+=item Build.PL
+
+=item dist.ini
+
+=item L<cpanfile>
+
+=back
+
 =cut
 
+sub dist_indicator_file_list {
+    qw{Makefile.PL Build.PL dist.ini cpanfile};
+}
+
 sub home {
     my $class = shift;
 
@@ -168,9 +202,8 @@ sub home {
             # pop off /lib and /blib if they're there
             $home = $home->parent while $home =~ /b?lib$/;
 
-            # only return the dir if it has a Makefile.PL or Build.PL
-            if (-f $home->file("Makefile.PL") or -f $home->file("Build.PL")) {
-
+            # only return the dir if it has a Makefile.PL or Build.PL or dist.ini
+            if (grep { -f $home->file($_) } dist_indicator_file_list()) {
                 # clean up relative path:
                 # MyApp/script/.. -> MyApp
 
@@ -295,7 +328,7 @@ sub merge_hashes {
     my ( $lefthash, $righthash ) = @_;
 
     return $lefthash unless defined $righthash;
-    
+
     my %merged = %$lefthash;
     for my $key ( keys %$righthash ) {
         my $right_ref = ( ref $righthash->{ $key } || '' ) eq 'HASH';
@@ -309,7 +342,7 @@ sub merge_hashes {
             $merged{ $key } = $righthash->{ $key };
         }
     }
-    
+
     return \%merged;
 }
 
@@ -344,10 +377,10 @@ All you need to get this work, is:
 
 1) Install Term::Size::Any, or
 
-2) Export $COLUMNS from your shell. 
+2) Export $COLUMNS from your shell.
 
 (Warning to bash users: 'echo $COLUMNS' may be showing you the bash
-variable, not $ENV{COLUMNS}. 'export COLUMNS=$COLUMNS' and you should see 
+variable, not $ENV{COLUMNS}. 'export COLUMNS=$COLUMNS' and you should see
 that 'env' now lists COLUMNS.)
 
 As last resort, default value of 80 chars will be used.
@@ -375,13 +408,89 @@ sub term_width {
     return $_term_width = $width;
 }
 
+
+=head2 resolve_namespace
+
+Method which adds the namespace for plugins and actions.
+
+  __PACKAGE__->setup(qw(MyPlugin));
+
+  # will load Catalyst::Plugin::MyPlugin
+
+=cut
+
+
+sub resolve_namespace {
+    my $appnamespace = shift;
+    my $namespace = shift;
+    my @classes = @_;
+    return String::RewritePrefix->rewrite({
+        q[]  => qq[${namespace}::],
+        q[+] => q[],
+        (defined $appnamespace
+            ? (q[~] => qq[${appnamespace}::])
+            : ()
+        ),
+    }, @classes);
+}
+
+=head2 build_middleware (@args)
+
+Internal application that converts a single middleware definition (see
+L<Catalyst/psgi_middleware>) into an actual instance of middleware.
+
+=cut
+
+sub build_middleware {
+    my ($class, $namespace, @init_args) = @_;
+
+    if(
+      $namespace =~s/^\+// ||
+      $namespace =~/^Plack::Middleware/ ||
+      $namespace =~/^$class/
+    ) {  ## the string is a full namespace
+        return Class::Load::try_load_class($namespace) ?
+          $namespace->new(@init_args) :
+            die "Can't load class $namespace";
+    } else { ## the string is a partial namespace
+        if(Class::Load::try_load_class("Plack::Middleware::$namespace")) { ## Act like Plack::Builder
+            return "Plack::Middleware::$namespace"->new(@init_args);
+        } elsif(Class::Load::try_load_class("$class::$namespace")) { ## Load Middleware from Project namespace
+            return "$class::$namespace"->new(@init_args);
+        }
+    }
+
+    return; ## be sure we can count on a proper return when valid
+}
+
+=head2 apply_registered_middleware ($psgi)
+
+Given a $psgi reference, wrap all the L<Catalyst/registered_middlewares>
+around it and return the wrapped version.
+
+This exists to deal with the fact Catalyst registered middleware can be
+either an object with a wrap method or a coderef.
+
+=cut
+
+sub apply_registered_middleware {
+    my ($class, $psgi) = @_;
+    my $new_psgi = $psgi;
+    foreach my $middleware ($class->registered_middlewares) {
+        $new_psgi = Scalar::Util::blessed $middleware ?
+          $middleware->wrap($new_psgi) :
+            $middleware->($new_psgi);
+    }
+    return $new_psgi;
+}
+
 =head1 AUTHORS
 
 Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
-This program is free software, you can redistribute it and/or modify it under
+This library is free software. You can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =cut
